=== added file 'bin/ivle-fetchsubmissions'
--- bin/ivle-fetchsubmissions	1970-01-01 00:00:00 +0000
+++ bin/ivle-fetchsubmissions	2009-05-11 14:12:33 +0000
@@ -0,0 +1,246 @@
+#!/usr/bin/env python
+# IVLE - Informatics Virtual Learning Environment
+# Copyright (C) 2007-2009 The University of Melbourne
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+# Program: Fetch Submissions
+# Author:  Matt Giuca
+
+# Script to retrieve all submissions for a particular project.
+# Requires root to run.
+
+import sys
+import os
+import shutil
+import datetime
+import optparse
+import zipfile
+import traceback
+
+import pysvn
+
+if os.getuid() != 0:
+    print >>sys.stderr, "Must run %s as root." % os.path.basename(sys.argv[0])
+    sys.exit()
+
+import ivle.config
+import ivle.database
+
+from ivle.database import Project, ProjectSet, Offering, Subject
+
+def fetch_submission(submission, target, svnclient, config, zip=False):
+    """Fetch a submission from a user's repository, and dump it in a given
+    directory.
+    @param submission: Submission object, detailing the submission.
+    @param target: Target directory for the project (will create a
+        subdirectory for each submission).
+    @param svnclient: pysvn.Client object.
+    @param config: Config object.
+    @param zip: If True, zips up the submission.
+    """
+    # submission_name is the name of the user or group who owns the repo
+    submission_name = submission.assessed.principal.name
+    # target_final is the directory to place the files in
+    target_final = os.path.join(target, submission.assessed.principal.name)
+    if not os.path.exists(target_final):
+        os.makedirs(target_final)
+    url = get_repo_url(submission, config)
+    revision = pysvn.Revision(pysvn.opt_revision_kind.number,
+                              submission.revision)
+    svnclient.export(url, target_final, force=True,
+        revision=revision, recurse=True)
+
+    # If required, zip up the directory
+    if zip:
+        make_zip(target_final, target_final + ".zip")
+        # Remove the target tree
+        shutil.rmtree(target_final)
+
+def get_repo_url(submission, config):
+    """Gets a local (file:) URL to the repository for a given submission.
+    This will consult submission.path to find the path within the repository
+    to check out.
+    @param submission: Submission object, detailing the submission.
+    @param config: Config object.
+    """
+    # NOTE: This code is mostly copied from services/usrmgt-server
+    if submission.assessed.is_group:
+        # The offering this group is in
+        offering = submission.assessed.project.project_set.offering
+        groupname = submission.assessed.principal.name
+        # The name of the repository directory within 'groups' is
+        # SUBJECT_YEAR_SEMESTER_GROUP
+        namespace = "_".join([offering.subject.short_name,
+            offering.semester.year, offering.semester.semester, groupname])
+        repo_path = os.path.join(config['paths']['svn']['repo_path'],
+                                'groups', namespace)
+    else:
+        # The name of the repository directory within 'users' is the username
+        username = submission.assessed.principal.name
+        repo_path = os.path.join(config['paths']['svn']['repo_path'],
+                                'users', username)
+
+    path_in_repo = submission.path
+    # Change an absolute path into a relative one (to the top of SVN)
+    if path_in_repo[:1] == os.sep or path_in_repo[:1] == os.altsep:
+        path_in_repo = path_in_repo[1:]
+
+    # Attach "file://" to the front of the absolute path, to make it a URL
+    return "file://" + os.path.join(os.path.abspath(repo_path), path_in_repo)
+
+def make_zip(source, dest):
+    """Zip up a directory tree or file. The ZIP file will always contain just
+    a single directory or file at the top level (it will not be a ZIP bomb).
+    @param source: Path to a directory or file to zip up.
+    @param dest: Path to a zip file to create.
+    """
+    # NOTE: This code is mostly copied from ivle.zip (but very different)
+    zip = zipfile.ZipFile(dest, 'w')
+
+    # Write the source file/directory itself
+    # (If this is a directory it will NOT recurse)
+    zip.write(source, os.path.basename(source))
+
+    if os.path.isdir(source):
+        # All paths within the zip file are relative to relativeto
+        relativeto = os.path.dirname(source)
+        # Write the top-level directory
+        # Walk the directory tree
+        def error(err):
+            raise OSError("Could not access a file (zipping)")
+        for (dirpath, dirnames, filenames) in \
+            os.walk(source, onerror=error):
+            arc_dirpath = os.path.relpath(dirpath, relativeto)
+            for filename in dirnames + filenames:
+                zip.write(os.path.join(dirpath, filename),
+                            os.path.join(arc_dirpath, filename))
+
+    zip.close()
+
+def main(argv=None):
+    global store
+    if argv is None:
+        argv = sys.argv
+
+    usage = """usage: %prog [options] subject projname
+    (requires root)
+    Retrieves all submissions for a given project. Places each submission in
+    its own directory, in a subdirectory of '.'. Any errors are reported to
+    stderr (otherwise is silent).
+    subject/projname is the subject/project's short name.
+    """
+
+    # Parse arguments
+    parser = optparse.OptionParser(usage)
+    parser.add_option("-s", "--semester",
+        action="store", dest="semester", metavar="YEAR/SEMESTER",
+        help="Semester of the subject's offering (eg. 2009/1). "
+             "Defaults to the currently active semester.",
+        default=None)
+    parser.add_option("-d", "--dest",
+        action="store", dest="dest", metavar="PATH",
+        help="Destination directory (default to '.', creates a subdirectory, "
+            "so will not pollute PATH).",
+        default=".")
+    parser.add_option("-z", "--zip",
+        action="store_true", dest="zip",
+        help="Store each submission in a Zip file.",
+        default=False)
+    (options, args) = parser.parse_args(argv[1:])
+
+    if len(args) < 2:
+        parser.print_help()
+        parser.exit()
+
+    subject_name = unicode(args[0])
+    project_name = unicode(args[1])
+
+    if options.semester is None:
+        year, semester = None, None
+    else:
+        try:
+            year, semester = options.semester.split('/')
+            if len(year) == 0 or len(semester) == 0:
+                raise ValueError()
+        except ValueError:
+            parser.error('Invalid semester (must have form "year/semester")')
+
+    svnclient = pysvn.Client()
+    config = ivle.config.Config(plugins=False)
+    store = ivle.database.get_store(config)
+
+    # Get the subject from the DB
+    subject = store.find(Subject,
+                     Subject.short_name == subject_name).one()
+    if subject is None:
+        print >>sys.stderr, "No subject with short name '%s'" % subject_name
+        return 1
+
+    # Get the offering from the DB
+    if semester is None:
+        # None specified - get the current offering from the DB
+        offerings = list(subject.active_offerings())
+        if len(offerings) == 0:
+            print >>sys.stderr, ("No active offering for subject '%s'"
+                                 % subject_name)
+            return 1
+        elif len(offerings) > 1:
+            print >>sys.stderr, ("Multiple active offerings for subject '%s':"
+                                 % subject_name)
+            print >>sys.stderr, "Please use one of:"
+            for offering in offerings:
+                print >>sys.stderr, ("    --semester=%s/%s"
+                    % (offering.semester.year, offering.semester.semester))
+            return 1
+        else:
+            offering = offerings[0]
+    else:
+        # Get the offering for the specified semester
+        offering = subject.offering_for_semester(year, semester)
+        if offering is None:
+            print >>sys.stderr, (
+                "No offering for subject '%s' in semester %s/%s"
+                % (subject_name, year, semester))
+            return 1
+
+    # Get the project from the DB
+    project = store.find(Project,
+                         Project.project_set_id == ProjectSet.id,
+                         ProjectSet.offering == offering,
+                         Project.short_name == project_name).one()
+    if project is None:
+        print >>sys.stderr, "No project with short name '%s'" % project_name
+        return 1
+
+    # Target directory is DEST/subject/year/semester/project
+    target_dir = os.path.join(options.dest, subject_name,
+        offering.semester.year, offering.semester.semester, project_name)
+    if not os.path.exists(target_dir):
+        os.makedirs(target_dir)
+
+    for submission in project.latest_submissions:
+        try:
+            fetch_submission(submission, target_dir, svnclient, config,
+                             zip=options.zip)
+        except Exception, e:
+            # Catch all exceptions (to ensure if one student has a problem, it
+            # is reported, and we can continue)
+            print >>sys.stderr, "ERROR on submission for %s:" % (
+                submission.assessed.principal.display_name)
+            traceback.print_exc(e)
+
+if __name__ == "__main__":
+    sys.exit(main(sys.argv))

=== modified file 'ivle/database.py'
--- ivle/database.py	2009-05-06 01:32:44 +0000
+++ ivle/database.py	2009-05-11 08:29:59 +0000
@@ -28,6 +28,7 @@
 
 from storm.locals import create_database, Store, Int, Unicode, DateTime, \
                          Reference, ReferenceSet, Bool, Storm, Desc
+from storm.expr import Select, Max
 from storm.exceptions import NotOneError, IntegrityError
 
 from ivle.worksheet.rst import rst
@@ -379,9 +380,16 @@
 
         return enrolment
 
-    def get_students(self):
-        enrolments = self.enrolments.find(role=u'student')
-        return [enrolment.user for enrolment in enrolments]
+    def get_members_by_role(self, role):
+        return Store.of(self).find(User,
+                Enrolment.user_id == User.id,
+                Enrolment.offering_id == self.id,
+                Enrolment.role == role
+                )
+
+    @property
+    def students(self):
+        return self.get_members_by_role(u'student')
 
 class Enrolment(Storm):
     """An enrolment of a user in an offering.
@@ -442,11 +450,15 @@
     def get_permissions(self, user):
         return self.offering.get_permissions(user)
 
-    # Get the individuals (groups or users) Assigned to this project
-    def get_assigned(self):
-        #If its a Solo project, return everyone in offering
+    @property
+    def assigned(self):
+        """Get the entities (groups or users) assigned to submit this project.
+
+        This will be a Storm ResultSet.
+        """
+        #If its a solo project, return everyone in offering
         if self.max_students_per_group is None:
-            return self.offering.get_students()
+            return self.offering.students
         else:
             return self.project_groups
 
@@ -506,6 +518,19 @@
     def get_permissions(self, user):
         return self.project_set.offering.get_permissions(user)
 
+    @property
+    def latest_submissions(self):
+        """Return the latest submission for each Assessed."""
+        return Store.of(self).find(ProjectSubmission,
+            Assessed.project_id == self.id,
+            ProjectSubmission.assessed_id == Assessed.id,
+            ProjectSubmission.date_submitted == Select(
+                    Max(ProjectSubmission.date_submitted),
+                    ProjectSubmission.assessed_id == Assessed.id,
+                    tables=ProjectSubmission
+            )
+        )
+
 
 class ProjectGroup(Storm):
     """A group of students working together on a project."""
@@ -605,6 +630,11 @@
             self.user or self.project_group, self.project)
 
     @property
+    def is_group(self):
+        """True if the Assessed is a group, False if it is a user."""
+        return self.project_group is not None
+
+    @property
     def principal(self):
         return self.project_group or self.user
 

=== modified file 'ivle/webapp/admin/subject.py'
--- ivle/webapp/admin/subject.py	2009-05-06 01:32:44 +0000
+++ ivle/webapp/admin/subject.py	2009-05-12 08:43:02 +0000
@@ -23,7 +23,9 @@
 # A sample / testing application for IVLE.
 
 import os
+import os.path
 import urllib
+import urlparse
 import cgi
 
 from storm.locals import Desc
@@ -218,19 +220,30 @@
         if self.context is None:
             raise NotFound()
 
+    def build_subversion_url(self, svnroot, submission):
+        princ = submission.assessed.principal
+
+        if isinstance(princ, User):
+            path = 'users/%s' % princ.login
+        else:
+            path = 'groups/%s_%s_%s_%s' % (
+                    princ.project_set.offering.subject.short_name,
+                    princ.project_set.offering.semester.year,
+                    princ.project_set.offering.semester.semester,
+                    princ.name
+                    )
+        return urlparse.urljoin(
+                    svnroot,
+                    os.path.join(path, submission.path[1:] if
+                                       submission.path.startswith(os.sep) else
+                                       submission.path))
+
     def populate(self, req, ctx):
         ctx['format_datetime_short'] = ivle.date.format_datetime_for_paragraph
-
+        ctx['build_subversion_url'] = self.build_subversion_url
+        ctx['svn_addr'] = req.config['urls']['svn_addr']
         ctx['project'] = self.context
-        ctx['assesseds'] = self.context.assesseds
-
-        ctx['submissions'] = []
-        for assessed in self.context.assesseds:
-            if assessed.submissions.count() > 0:
-                ctx['submissions'].append(
-                        assessed.submissions.order_by(
-                            ProjectSubmission.date_submitted)[-1])
-        ctx['assigned'] = self.context.project_set.get_assigned()
+        ctx['user'] = req.user
 
 class Plugin(ViewPlugin, MediaPlugin):
     urls = [

=== modified file 'ivle/webapp/admin/templates/project.html'
--- ivle/webapp/admin/templates/project.html	2009-05-06 03:32:21 +0000
+++ ivle/webapp/admin/templates/project.html	2009-05-12 08:43:02 +0000
@@ -7,27 +7,27 @@
     <h1>${project.name} for ${project.project_set.offering.subject.name}</h1>
     <div id="ivle_padding">
       <p>${project.synopsis}</p>
-      <p>Submissions: ${len(submissions)}/${len(assigned)}</p>
-      <table border="1" summary="Latest Submissions" 
-          py:if="len(submissions) > 0">
+      <p>Submissions: ${project.latest_submissions.count()}/${project.project_set.assigned.count()}</p>
+      <table border="1" summary="Latest Submissions"
+             py:if="project.latest_submissions.count() > 0">
         <tr>
-          <th>Name</th>
-          <th>Submitted By</th>
-          <th>Time Submitted</th>
-          <th>Repository</th>
+          <th>Who</th>
+          <th>By</th>
+          <th>Date</th>
+          <th>Download</th>
         </tr>
-      <py:for each="submission in submissions">
+      <py:for each="submission in project.latest_submissions">
         <tr>
           <td>${submission.assessed.principal.display_name}</td>
           <td>${submission.submitter.display_name}</td>
-          <td>${submission.date_submitted}</td>
-          <td>${submission.path}</td>
+          <td>${submission.date_submitted.strftime("%Y-%m-%d %H:%M:%S")}</td>
+          <td>svn co --username ${user.login} -r${submission.revision} ${build_subversion_url(svn_addr, submission)}</td>
         </tr>
       </py:for>
       </table>
       <table>
         <tr><th>Assigned</th></tr>
-        <py:for each="assignee in assigned">
+        <py:for each="assignee in project.project_set.assigned">
           <tr>
             <td>${assignee.display_name}</td>
             <td><a>Grant an extension</a></td>

=== modified file 'ivle/webapp/admin/user.py'
--- ivle/webapp/admin/user.py	2009-02-25 08:21:38 +0000
+++ ivle/webapp/admin/user.py	2009-05-11 04:21:11 +0000
@@ -55,7 +55,7 @@
         return user
 
 class UserSettingsView(XHTMLView):
-    template = 'user-settings.html'
+    template = 'templates/user-settings.html'
     tab = 'settings'
     permission = 'edit'
 

=== modified file 'setup/util.py'
--- setup/util.py	2009-05-11 04:51:08 +0000
+++ setup/util.py	2009-05-12 08:44:07 +0000
@@ -312,6 +312,7 @@
     list_user_binaries = [
         "bin/ivle-enrol",
         "bin/ivle-enrolallusers",
+        "bin/ivle-fetchsubmissions",
         "bin/ivle-listusers",
         "bin/ivle-makeuser",
         "bin/ivle-marks",

